/* Generated by re2c 3.0 */
#line 1 "ext/rbs_extension/lexer.re"
#include "rbs_extension.h"

token rbsparser_next_token(lexstate *state) {
  lexstate backup;

start:
  backup = *state;

  
#line 13 "ext/rbs_extension/lexer.c"
{
	unsigned int yych;
	unsigned int yyaccept = 0;
	yych = peek(state);
	switch (yych) {
		case 0x00000000: goto yy1;
		case '\t':
		case '\n':
		case '\r':
		case ' ': goto yy4;
		case '!': goto yy6;
		case '"': goto yy8;
		case '#': goto yy9;
		case '$': goto yy11;
		case '%': goto yy12;
		case '&': goto yy13;
		case '\'': goto yy14;
		case '(': goto yy15;
		case ')': goto yy16;
		case '*': goto yy17;
		case '+': goto yy18;
		case ',': goto yy19;
		case '-': goto yy20;
		case '.': goto yy21;
		case '/':
		case '~': goto yy23;
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9': goto yy24;
		case ':': goto yy26;
		case '<': goto yy28;
		case '=': goto yy30;
		case '>': goto yy32;
		case '?': goto yy33;
		case '@': goto yy34;
		case 'A':
		case 'B':
		case 'C':
		case 'D':
		case 'E':
		case 'F':
		case 'G':
		case 'H':
		case 'I':
		case 'J':
		case 'K':
		case 'L':
		case 'M':
		case 'N':
		case 'O':
		case 'P':
		case 'Q':
		case 'R':
		case 'S':
		case 'T':
		case 'U':
		case 'V':
		case 'W':
		case 'X':
		case 'Y':
		case 'Z': goto yy35;
		case '[': goto yy37;
		case ']': goto yy38;
		case '^': goto yy39;
		case '_': goto yy40;
		case '`': goto yy42;
		case 'a': goto yy44;
		case 'b': goto yy46;
		case 'c': goto yy47;
		case 'd': goto yy48;
		case 'e': goto yy49;
		case 'f': goto yy50;
		case 'g':
		case 'h':
		case 'j':
		case 'k':
		case 'l':
		case 'q':
		case 'r':
		case 'w':
		case 'x':
		case 'y':
		case 'z': goto yy51;
		case 'i': goto yy53;
		case 'm': goto yy54;
		case 'n': goto yy55;
		case 'o': goto yy56;
		case 'p': goto yy57;
		case 's': goto yy58;
		case 't': goto yy59;
		case 'u': goto yy60;
		case 'v': goto yy61;
		case '{': goto yy62;
		case '|': goto yy63;
		case '}': goto yy64;
		default: goto yy2;
	}
yy1:
	skip(state);
#line 138 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pEOF); }
#line 122 "ext/rbs_extension/lexer.c"
yy2:
	skip(state);
yy3:
#line 139 "ext/rbs_extension/lexer.re"
	{ return next_token(state, ErrorToken); }
#line 128 "ext/rbs_extension/lexer.c"
yy4:
	skip(state);
	yych = peek(state);
	if (yych <= '\f') {
		if (yych <= 0x00000008) goto yy5;
		if (yych <= '\n') goto yy4;
	} else {
		if (yych <= '\r') goto yy4;
		if (yych == ' ') goto yy4;
	}
yy5:
#line 137 "ext/rbs_extension/lexer.re"
	{ state->start = state->current; goto start; }
#line 142 "ext/rbs_extension/lexer.c"
yy6:
	skip(state);
	yych = peek(state);
	if (yych == '=') goto yy23;
	if (yych == '~') goto yy23;
yy7:
#line 49 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tOPERATOR); }
#line 151 "ext/rbs_extension/lexer.c"
yy8:
	yyaccept = 0;
	skip(state);
	backup = *state;
	yych = peek(state);
	if (yych <= 0x00000000) goto yy3;
	goto yy66;
yy9:
	skip(state);
	yych = peek(state);
	if (yych <= 0x00000000) goto yy10;
	if (yych != '\n') goto yy9;
yy10:
#line 60 "ext/rbs_extension/lexer.re"
	{
        return next_token(
          state,
          state->first_token_of_line ? tLINECOMMENT : tCOMMENT
        );
      }
#line 172 "ext/rbs_extension/lexer.c"
yy11:
	skip(state);
	yych = peek(state);
	if (yych <= ')') {
		if (yych <= 0x0000001F) {
			if (yych <= '\n') {
				if (yych <= 0x00000000) goto yy3;
				if (yych <= 0x00000008) goto yy70;
				goto yy3;
			} else {
				if (yych == '\r') goto yy3;
				goto yy70;
			}
		} else {
			if (yych <= '#') {
				if (yych <= ' ') goto yy3;
				if (yych <= '"') goto yy72;
				goto yy70;
			} else {
				if (yych == '%') goto yy3;
				if (yych <= '\'') goto yy72;
				goto yy3;
			}
		}
	} else {
		if (yych <= 'Z') {
			if (yych <= '/') {
				if (yych == '-') goto yy70;
				goto yy72;
			} else {
				if (yych <= '9') goto yy70;
				if (yych <= '>') goto yy72;
				goto yy70;
			}
		} else {
			if (yych <= '^') {
				if (yych == '\\') goto yy72;
				goto yy3;
			} else {
				if (yych <= 'z') goto yy70;
				if (yych <= '}') goto yy3;
				if (yych <= '~') goto yy72;
				goto yy70;
			}
		}
	}
yy12:
	yyaccept = 1;
	skip(state);
	backup = *state;
	yych = peek(state);
	if (yych == 'a') goto yy73;
	goto yy7;
yy13:
	skip(state);
#line 34 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pAMP); }
#line 230 "ext/rbs_extension/lexer.c"
yy14:
	yyaccept = 0;
	skip(state);
	backup = *state;
	yych = peek(state);
	if (yych <= 0x00000000) goto yy3;
	goto yy75;
yy15:
	skip(state);
#line 25 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pLPAREN); }
#line 242 "ext/rbs_extension/lexer.c"
yy16:
	skip(state);
#line 26 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pRPAREN); }
#line 247 "ext/rbs_extension/lexer.c"
yy17:
	skip(state);
	yych = peek(state);
	if (yych == '*') goto yy79;
#line 36 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pSTAR); }
#line 254 "ext/rbs_extension/lexer.c"
yy18:
	skip(state);
	yych = peek(state);
	if (yych <= '/') goto yy7;
	if (yych <= '9') goto yy24;
	if (yych == '@') goto yy23;
	goto yy7;
yy19:
	skip(state);
#line 31 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pCOMMA); }
#line 266 "ext/rbs_extension/lexer.c"
yy20:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') goto yy7;
		if (yych <= '9') goto yy24;
		goto yy7;
	} else {
		if (yych <= '>') goto yy80;
		if (yych == '@') goto yy23;
		goto yy7;
	}
yy21:
	yyaccept = 2;
	skip(state);
	backup = *state;
	yych = peek(state);
	if (yych == '.') goto yy81;
yy22:
#line 38 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pDOT); }
#line 288 "ext/rbs_extension/lexer.c"
yy23:
	skip(state);
	goto yy7;
yy24:
	skip(state);
	yych = peek(state);
	if (yych <= '/') goto yy25;
	if (yych <= '9') goto yy24;
	if (yych == '_') goto yy24;
yy25:
#line 52 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tINTEGER); }
#line 301 "ext/rbs_extension/lexer.c"
yy26:
	yyaccept = 3;
	skip(state);
	backup = *state;
	yych = peek(state);
	switch (yych) {
		case '!': goto yy82;
		case '"': goto yy84;
		case '$': goto yy85;
		case '%':
		case '&':
		case '/':
		case '^':
		case '`':
		case '|':
		case '~': goto yy86;
		case '\'': goto yy87;
		case '*': goto yy88;
		case '+':
		case '-': goto yy89;
		case ':': goto yy90;
		case '<': goto yy91;
		case '=': goto yy92;
		case '>': goto yy93;
		case '@': goto yy94;
		case 'A':
		case 'B':
		case 'C':
		case 'D':
		case 'E':
		case 'F':
		case 'G':
		case 'H':
		case 'I':
		case 'J':
		case 'K':
		case 'L':
		case 'M':
		case 'N':
		case 'O':
		case 'P':
		case 'Q':
		case 'R':
		case 'S':
		case 'T':
		case 'U':
		case 'V':
		case 'W':
		case 'X':
		case 'Y':
		case 'Z':
		case '_':
		case 'a':
		case 'b':
		case 'c':
		case 'd':
		case 'e':
		case 'f':
		case 'g':
		case 'h':
		case 'i':
		case 'j':
		case 'k':
		case 'l':
		case 'm':
		case 'n':
		case 'o':
		case 'p':
		case 'q':
		case 'r':
		case 's':
		case 't':
		case 'u':
		case 'v':
		case 'w':
		case 'x':
		case 'y':
		case 'z': goto yy95;
		case '[': goto yy97;
		default: goto yy27;
	}
yy27:
#line 45 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pCOLON); }
#line 386 "ext/rbs_extension/lexer.c"
yy28:
	skip(state);
	yych = peek(state);
	if (yych <= ';') goto yy29;
	if (yych <= '<') goto yy23;
	if (yych <= '=') goto yy98;
yy29:
#line 47 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pLT); }
#line 396 "ext/rbs_extension/lexer.c"
yy30:
	skip(state);
	yych = peek(state);
	if (yych <= '>') {
		if (yych <= '<') goto yy31;
		if (yych <= '=') goto yy99;
		goto yy100;
	} else {
		if (yych == '~') goto yy23;
	}
yy31:
#line 44 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pEQ); }
#line 410 "ext/rbs_extension/lexer.c"
yy32:
	skip(state);
	yych = peek(state);
	if (yych <= '<') goto yy7;
	if (yych <= '>') goto yy23;
	goto yy7;
yy33:
	skip(state);
#line 35 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pQUESTION); }
#line 421 "ext/rbs_extension/lexer.c"
yy34:
	yyaccept = 0;
	skip(state);
	backup = *state;
	yych = peek(state);
	if (yych <= '^') {
		if (yych <= '?') goto yy3;
		if (yych <= '@') goto yy101;
		if (yych <= 'Z') goto yy102;
		goto yy3;
	} else {
		if (yych == '`') goto yy3;
		if (yych <= 'z') goto yy102;
		goto yy3;
	}
yy35:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy35;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy36;
			if (yych <= 'Z') goto yy35;
		} else {
			if (yych == '`') goto yy36;
			if (yych <= 'z') goto yy35;
		}
	}
yy36:
#line 123 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tUIDENT); }
#line 459 "ext/rbs_extension/lexer.c"
yy37:
	skip(state);
	yych = peek(state);
	if (yych == ']') goto yy106;
#line 27 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pLBRACKET); }
#line 466 "ext/rbs_extension/lexer.c"
yy38:
	skip(state);
#line 28 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pRBRACKET); }
#line 471 "ext/rbs_extension/lexer.c"
yy39:
	skip(state);
#line 33 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pHAT); }
#line 476 "ext/rbs_extension/lexer.c"
yy40:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy107;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy41;
			if (yych <= 'Z') goto yy109;
		} else {
			if (yych == '`') goto yy41;
			if (yych <= 'z') goto yy107;
		}
	}
yy41:
#line 126 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tULLIDENT); }
#line 499 "ext/rbs_extension/lexer.c"
yy42:
	yyaccept = 4;
	skip(state);
	backup = *state;
	yych = peek(state);
	if (yych <= ' ') {
		if (yych <= 0x00000000) goto yy43;
		if (yych <= 0x0000001F) goto yy111;
	} else {
		if (yych != ':') goto yy111;
	}
yy43:
#line 40 "ext/rbs_extension/lexer.re"
	{  return next_token(state, tOPERATOR); }
#line 514 "ext/rbs_extension/lexer.c"
yy44:
	skip(state);
	yych = peek(state);
	if (yych == 'l') goto yy112;
	if (yych == 't') goto yy113;
	goto yy52;
yy45:
#line 122 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tLIDENT); }
#line 524 "ext/rbs_extension/lexer.c"
yy46:
	skip(state);
	yych = peek(state);
	if (yych == 'o') goto yy114;
	goto yy52;
yy47:
	skip(state);
	yych = peek(state);
	if (yych == 'l') goto yy115;
	goto yy52;
yy48:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy116;
	goto yy52;
yy49:
	skip(state);
	yych = peek(state);
	if (yych == 'n') goto yy117;
	if (yych == 'x') goto yy118;
	goto yy52;
yy50:
	skip(state);
	yych = peek(state);
	if (yych == 'a') goto yy119;
	goto yy52;
yy51:
	skip(state);
	yych = peek(state);
yy52:
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
			goto yy45;
		} else {
			if (yych <= '9') goto yy51;
			if (yych <= '<') goto yy45;
			goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy45;
			if (yych <= 'Z') goto yy51;
			goto yy45;
		} else {
			if (yych == '`') goto yy45;
			if (yych <= 'z') goto yy51;
			goto yy45;
		}
	}
yy53:
	skip(state);
	yych = peek(state);
	if (yych == 'n') goto yy120;
	goto yy52;
yy54:
	skip(state);
	yych = peek(state);
	if (yych == 'o') goto yy122;
	goto yy52;
yy55:
	skip(state);
	yych = peek(state);
	if (yych == 'i') goto yy123;
	goto yy52;
yy56:
	skip(state);
	yych = peek(state);
	if (yych == 'u') goto yy124;
	goto yy52;
yy57:
	skip(state);
	yych = peek(state);
	if (yych == 'r') goto yy125;
	if (yych == 'u') goto yy126;
	goto yy52;
yy58:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy127;
	if (yych == 'i') goto yy128;
	goto yy52;
yy59:
	skip(state);
	yych = peek(state);
	if (yych <= 'q') {
		if (yych == 'o') goto yy129;
		goto yy52;
	} else {
		if (yych <= 'r') goto yy130;
		if (yych == 'y') goto yy131;
		goto yy52;
	}
yy60:
	skip(state);
	yych = peek(state);
	if (yych == 'n') goto yy132;
	goto yy52;
yy61:
	skip(state);
	yych = peek(state);
	if (yych == 'o') goto yy133;
	goto yy52;
yy62:
	skip(state);
#line 29 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pLBRACE); }
#line 632 "ext/rbs_extension/lexer.c"
yy63:
	skip(state);
#line 32 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pBAR); }
#line 637 "ext/rbs_extension/lexer.c"
yy64:
	skip(state);
#line 30 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pRBRACE); }
#line 642 "ext/rbs_extension/lexer.c"
yy65:
	skip(state);
	yych = peek(state);
yy66:
	if (yych <= '"') {
		if (yych <= 0x00000000) goto yy67;
		if (yych <= '!') goto yy65;
		goto yy68;
	} else {
		if (yych == '\\') goto yy69;
		goto yy65;
	}
yy67:
	*state = backup;
	if (yyaccept <= 3) {
		if (yyaccept <= 1) {
			if (yyaccept == 0) {
				goto yy3;
			} else {
				goto yy7;
			}
		} else {
			if (yyaccept == 2) {
				goto yy22;
			} else {
				goto yy27;
			}
		}
	} else {
		if (yyaccept <= 5) {
			if (yyaccept == 4) {
				goto yy43;
			} else {
				goto yy77;
			}
		} else {
			goto yy147;
		}
	}
yy68:
	skip(state);
#line 100 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tDQSTRING); }
#line 686 "ext/rbs_extension/lexer.c"
yy69:
	skip(state);
	yych = peek(state);
	if (yych <= 'd') {
		if (yych <= '[') {
			if (yych == '"') goto yy65;
			goto yy67;
		} else {
			if (yych <= '\\') goto yy65;
			if (yych <= '`') goto yy67;
			if (yych <= 'b') goto yy65;
			goto yy67;
		}
	} else {
		if (yych <= 'q') {
			if (yych <= 'f') goto yy65;
			if (yych == 'n') goto yy65;
			goto yy67;
		} else {
			if (yych == 'u') goto yy67;
			if (yych <= 'v') goto yy65;
			goto yy67;
		}
	}
yy70:
	skip(state);
	yych = peek(state);
	if (yych <= ',') {
		if (yych <= '\f') {
			if (yych <= 0x00000000) goto yy71;
			if (yych <= 0x00000008) goto yy70;
			if (yych >= '\v') goto yy70;
		} else {
			if (yych <= 0x0000001F) {
				if (yych >= 0x0000000E) goto yy70;
			} else {
				if (yych == '#') goto yy70;
			}
		}
	} else {
		if (yych <= '>') {
			if (yych <= '-') goto yy70;
			if (yych <= '/') goto yy71;
			if (yych <= '9') goto yy70;
		} else {
			if (yych <= '^') {
				if (yych <= 'Z') goto yy70;
			} else {
				if (yych <= 'z') goto yy70;
				if (yych >= 0x0000007F) goto yy70;
			}
		}
	}
yy71:
#line 133 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tGIDENT); }
#line 743 "ext/rbs_extension/lexer.c"
yy72:
	skip(state);
	goto yy71;
yy73:
	skip(state);
	yych = peek(state);
	if (yych <= 'Z') {
		if (yych <= '(') {
			if (yych <= '\'') goto yy67;
			goto yy134;
		} else {
			if (yych == '<') goto yy135;
			goto yy67;
		}
	} else {
		if (yych <= 'z') {
			if (yych <= '[') goto yy136;
			goto yy67;
		} else {
			if (yych <= '{') goto yy137;
			if (yych <= '|') goto yy138;
			goto yy67;
		}
	}
yy74:
	skip(state);
	yych = peek(state);
yy75:
	if (yych <= '\'') {
		if (yych <= 0x00000000) goto yy67;
		if (yych <= '&') goto yy74;
	} else {
		if (yych == '\\') goto yy78;
		goto yy74;
	}
yy76:
	skip(state);
yy77:
#line 101 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tSQSTRING); }
#line 784 "ext/rbs_extension/lexer.c"
yy78:
	skip(state);
	yych = peek(state);
	if (yych <= '\'') {
		if (yych <= 0x00000000) goto yy67;
		if (yych <= '&') goto yy74;
		goto yy139;
	} else {
		if (yych == '\\') goto yy78;
		goto yy74;
	}
yy79:
	skip(state);
#line 37 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pSTAR2); }
#line 800 "ext/rbs_extension/lexer.c"
yy80:
	skip(state);
#line 42 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pARROW); }
#line 805 "ext/rbs_extension/lexer.c"
yy81:
	skip(state);
	yych = peek(state);
	if (yych == '.') goto yy140;
	goto yy67;
yy82:
	skip(state);
	yych = peek(state);
	if (yych == '=') goto yy86;
	if (yych == '~') goto yy86;
yy83:
#line 120 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tSYMBOL); }
#line 819 "ext/rbs_extension/lexer.c"
yy84:
	skip(state);
	yych = peek(state);
	if (yych <= '"') {
		if (yych <= 0x00000000) goto yy67;
		if (yych <= '!') goto yy84;
		goto yy141;
	} else {
		if (yych == '\\') goto yy142;
		goto yy84;
	}
yy85:
	skip(state);
	yych = peek(state);
	if (yych <= ')') {
		if (yych <= 0x0000001F) {
			if (yych <= '\n') {
				if (yych <= 0x00000000) goto yy67;
				if (yych <= 0x00000008) goto yy143;
				goto yy67;
			} else {
				if (yych == '\r') goto yy67;
				goto yy143;
			}
		} else {
			if (yych <= '#') {
				if (yych <= ' ') goto yy67;
				if (yych <= '"') goto yy145;
				goto yy143;
			} else {
				if (yych == '%') goto yy67;
				if (yych <= '\'') goto yy145;
				goto yy67;
			}
		}
	} else {
		if (yych <= 'Z') {
			if (yych <= '/') {
				if (yych == '-') goto yy143;
				goto yy145;
			} else {
				if (yych <= '9') goto yy143;
				if (yych <= '>') goto yy145;
				goto yy143;
			}
		} else {
			if (yych <= '^') {
				if (yych == '\\') goto yy145;
				goto yy67;
			} else {
				if (yych <= 'z') goto yy143;
				if (yych <= '}') goto yy67;
				if (yych <= '~') goto yy145;
				goto yy143;
			}
		}
	}
yy86:
	skip(state);
	goto yy83;
yy87:
	skip(state);
	yych = peek(state);
	if (yych <= '\'') {
		if (yych <= 0x00000000) goto yy67;
		if (yych <= '&') goto yy87;
		goto yy146;
	} else {
		if (yych == '\\') goto yy148;
		goto yy87;
	}
yy88:
	skip(state);
	yych = peek(state);
	if (yych == '*') goto yy86;
	goto yy83;
yy89:
	skip(state);
	yych = peek(state);
	if (yych == '@') goto yy86;
	goto yy83;
yy90:
	skip(state);
#line 46 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pCOLON2); }
#line 905 "ext/rbs_extension/lexer.c"
yy91:
	skip(state);
	yych = peek(state);
	if (yych <= ';') goto yy83;
	if (yych <= '<') goto yy86;
	if (yych <= '=') goto yy149;
	goto yy83;
yy92:
	skip(state);
	yych = peek(state);
	if (yych == '=') goto yy150;
	if (yych == '~') goto yy86;
	goto yy67;
yy93:
	skip(state);
	yych = peek(state);
	if (yych <= '<') goto yy83;
	if (yych <= '>') goto yy86;
	goto yy83;
yy94:
	skip(state);
	yych = peek(state);
	if (yych <= '^') {
		if (yych <= '?') goto yy67;
		if (yych <= '@') goto yy151;
		if (yych <= 'Z') goto yy152;
		goto yy67;
	} else {
		if (yych == '`') goto yy67;
		if (yych <= 'z') goto yy152;
		goto yy67;
	}
yy95:
	skip(state);
	yych = peek(state);
	if (yych <= '>') {
		if (yych <= '/') {
			if (yych == '!') goto yy154;
		} else {
			if (yych <= '9') goto yy95;
			if (yych == '=') goto yy154;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '?') goto yy154;
			if (yych <= '@') goto yy96;
			if (yych <= 'Z') goto yy95;
		} else {
			if (yych == '`') goto yy96;
			if (yych <= 'z') goto yy95;
		}
	}
yy96:
#line 116 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tSYMBOL); }
#line 961 "ext/rbs_extension/lexer.c"
yy97:
	skip(state);
	yych = peek(state);
	if (yych == ']') goto yy150;
	goto yy67;
yy98:
	skip(state);
	yych = peek(state);
	if (yych == '>') goto yy23;
	goto yy7;
yy99:
	skip(state);
	yych = peek(state);
	if (yych == '=') goto yy23;
	goto yy7;
yy100:
	skip(state);
#line 43 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pFATARROW); }
#line 981 "ext/rbs_extension/lexer.c"
yy101:
	skip(state);
	yych = peek(state);
	if (yych <= '^') {
		if (yych <= '@') goto yy67;
		if (yych <= 'Z') goto yy155;
		goto yy67;
	} else {
		if (yych == '`') goto yy67;
		if (yych <= 'z') goto yy155;
		goto yy67;
	}
yy102:
	skip(state);
	yych = peek(state);
	if (yych <= 'Z') {
		if (yych <= '/') goto yy103;
		if (yych <= '9') goto yy102;
		if (yych >= 'A') goto yy102;
	} else {
		if (yych <= '_') {
			if (yych >= '_') goto yy102;
		} else {
			if (yych <= '`') goto yy103;
			if (yych <= 'z') goto yy102;
		}
	}
yy103:
#line 130 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tAIDENT); }
#line 1012 "ext/rbs_extension/lexer.c"
yy104:
	skip(state);
#line 127 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tBANGIDENT); }
#line 1017 "ext/rbs_extension/lexer.c"
yy105:
	skip(state);
#line 128 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tEQIDENT); }
#line 1022 "ext/rbs_extension/lexer.c"
yy106:
	skip(state);
	yych = peek(state);
	if (yych == '=') goto yy23;
#line 48 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pAREF_OPR); }
#line 1029 "ext/rbs_extension/lexer.c"
yy107:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy107;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy108;
			if (yych <= 'Z') goto yy107;
		} else {
			if (yych == '`') goto yy108;
			if (yych <= 'z') goto yy107;
		}
	}
yy108:
#line 124 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tULLIDENT); }
#line 1052 "ext/rbs_extension/lexer.c"
yy109:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy109;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy110;
			if (yych <= 'Z') goto yy109;
		} else {
			if (yych == '`') goto yy110;
			if (yych <= 'z') goto yy109;
		}
	}
yy110:
#line 125 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tULIDENT); }
#line 1075 "ext/rbs_extension/lexer.c"
yy111:
	skip(state);
	yych = peek(state);
	if (yych <= 0x00000000) goto yy67;
	if (yych == '`') goto yy157;
	goto yy111;
yy112:
	skip(state);
	yych = peek(state);
	if (yych == 'i') goto yy158;
	goto yy52;
yy113:
	skip(state);
	yych = peek(state);
	if (yych == 't') goto yy159;
	goto yy52;
yy114:
	skip(state);
	yych = peek(state);
	if (yych == 'o') goto yy160;
	if (yych == 't') goto yy161;
	goto yy52;
yy115:
	skip(state);
	yych = peek(state);
	if (yych == 'a') goto yy163;
	goto yy52;
yy116:
	skip(state);
	yych = peek(state);
	if (yych == 'f') goto yy164;
	goto yy52;
yy117:
	skip(state);
	yych = peek(state);
	if (yych == 'd') goto yy166;
	goto yy52;
yy118:
	skip(state);
	yych = peek(state);
	if (yych == 't') goto yy168;
	goto yy52;
yy119:
	skip(state);
	yych = peek(state);
	if (yych == 'l') goto yy169;
	goto yy52;
yy120:
	skip(state);
	yych = peek(state);
	if (yych <= '^') {
		if (yych <= '9') {
			if (yych == '!') goto yy104;
			if (yych >= '0') goto yy51;
		} else {
			if (yych <= '=') {
				if (yych >= '=') goto yy105;
			} else {
				if (yych <= '@') goto yy121;
				if (yych <= 'Z') goto yy51;
			}
		}
	} else {
		if (yych <= 'c') {
			if (yych == '`') goto yy121;
			if (yych <= 'b') goto yy51;
			goto yy170;
		} else {
			if (yych <= 's') {
				if (yych <= 'r') goto yy51;
				goto yy171;
			} else {
				if (yych <= 't') goto yy172;
				if (yych <= 'z') goto yy51;
			}
		}
	}
yy121:
#line 78 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kIN); }
#line 1156 "ext/rbs_extension/lexer.c"
yy122:
	skip(state);
	yych = peek(state);
	if (yych == 'd') goto yy173;
	goto yy52;
yy123:
	skip(state);
	yych = peek(state);
	if (yych == 'l') goto yy174;
	goto yy52;
yy124:
	skip(state);
	yych = peek(state);
	if (yych == 't') goto yy176;
	goto yy52;
yy125:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy178;
	if (yych == 'i') goto yy179;
	goto yy52;
yy126:
	skip(state);
	yych = peek(state);
	if (yych == 'b') goto yy180;
	goto yy52;
yy127:
	skip(state);
	yych = peek(state);
	if (yych == 'l') goto yy181;
	goto yy52;
yy128:
	skip(state);
	yych = peek(state);
	if (yych == 'n') goto yy182;
	goto yy52;
yy129:
	skip(state);
	yych = peek(state);
	if (yych == 'p') goto yy183;
	goto yy52;
yy130:
	skip(state);
	yych = peek(state);
	if (yych == 'u') goto yy185;
	goto yy52;
yy131:
	skip(state);
	yych = peek(state);
	if (yych == 'p') goto yy186;
	goto yy52;
yy132:
	skip(state);
	yych = peek(state);
	if (yych == 'c') goto yy187;
	if (yych == 't') goto yy188;
	goto yy52;
yy133:
	skip(state);
	yych = peek(state);
	if (yych == 'i') goto yy189;
	goto yy52;
yy134:
	skip(state);
	yych = peek(state);
	if (yych <= 0x00000000) goto yy67;
	if (yych == ')') goto yy190;
	goto yy134;
yy135:
	skip(state);
	yych = peek(state);
	if (yych <= 0x00000000) goto yy67;
	if (yych == '>') goto yy191;
	goto yy135;
yy136:
	skip(state);
	yych = peek(state);
	if (yych <= 0x00000000) goto yy67;
	if (yych == ']') goto yy192;
	goto yy136;
yy137:
	skip(state);
	yych = peek(state);
	if (yych <= 0x00000000) goto yy67;
	if (yych == '}') goto yy193;
	goto yy137;
yy138:
	skip(state);
	yych = peek(state);
	if (yych <= 0x00000000) goto yy67;
	if (yych == '|') goto yy194;
	goto yy138;
yy139:
	yyaccept = 5;
	skip(state);
	backup = *state;
	yych = peek(state);
	if (yych <= '\'') {
		if (yych <= 0x00000000) goto yy77;
		if (yych <= '&') goto yy74;
		goto yy76;
	} else {
		if (yych == '\\') goto yy78;
		goto yy74;
	}
yy140:
	skip(state);
#line 39 "ext/rbs_extension/lexer.re"
	{ return next_token(state, pDOT3); }
#line 1266 "ext/rbs_extension/lexer.c"
yy141:
	skip(state);
#line 102 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tDQSYMBOL); }
#line 1271 "ext/rbs_extension/lexer.c"
yy142:
	skip(state);
	yych = peek(state);
	if (yych <= 'd') {
		if (yych <= '[') {
			if (yych == '"') goto yy84;
			goto yy67;
		} else {
			if (yych <= '\\') goto yy84;
			if (yych <= '`') goto yy67;
			if (yych <= 'b') goto yy84;
			goto yy67;
		}
	} else {
		if (yych <= 'q') {
			if (yych <= 'f') goto yy84;
			if (yych == 'n') goto yy84;
			goto yy67;
		} else {
			if (yych == 'u') goto yy67;
			if (yych <= 'v') goto yy84;
			goto yy67;
		}
	}
yy143:
	skip(state);
	yych = peek(state);
	if (yych <= ',') {
		if (yych <= '\f') {
			if (yych <= 0x00000000) goto yy144;
			if (yych <= 0x00000008) goto yy143;
			if (yych >= '\v') goto yy143;
		} else {
			if (yych <= 0x0000001F) {
				if (yych >= 0x0000000E) goto yy143;
			} else {
				if (yych == '#') goto yy143;
			}
		}
	} else {
		if (yych <= '>') {
			if (yych <= '-') goto yy143;
			if (yych <= '/') goto yy144;
			if (yych <= '9') goto yy143;
		} else {
			if (yych <= '^') {
				if (yych <= 'Z') goto yy143;
			} else {
				if (yych <= 'z') goto yy143;
				if (yych >= 0x0000007F) goto yy143;
			}
		}
	}
yy144:
#line 119 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tSYMBOL); }
#line 1328 "ext/rbs_extension/lexer.c"
yy145:
	skip(state);
	goto yy144;
yy146:
	skip(state);
yy147:
#line 103 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tSQSYMBOL); }
#line 1337 "ext/rbs_extension/lexer.c"
yy148:
	skip(state);
	yych = peek(state);
	if (yych <= '\'') {
		if (yych <= 0x00000000) goto yy67;
		if (yych <= '&') goto yy87;
		goto yy195;
	} else {
		if (yych == '\\') goto yy148;
		goto yy87;
	}
yy149:
	skip(state);
	yych = peek(state);
	if (yych == '>') goto yy86;
	goto yy83;
yy150:
	skip(state);
	yych = peek(state);
	if (yych == '=') goto yy86;
	goto yy83;
yy151:
	skip(state);
	yych = peek(state);
	if (yych <= '^') {
		if (yych <= '@') goto yy67;
		if (yych <= 'Z') goto yy196;
		goto yy67;
	} else {
		if (yych == '`') goto yy67;
		if (yych <= 'z') goto yy196;
		goto yy67;
	}
yy152:
	skip(state);
	yych = peek(state);
	if (yych <= '>') {
		if (yych <= '/') {
			if (yych == '!') goto yy198;
		} else {
			if (yych <= '9') goto yy152;
			if (yych == '=') goto yy198;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '?') goto yy198;
			if (yych <= '@') goto yy153;
			if (yych <= 'Z') goto yy152;
		} else {
			if (yych == '`') goto yy153;
			if (yych <= 'z') goto yy152;
		}
	}
yy153:
#line 117 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tSYMBOL); }
#line 1394 "ext/rbs_extension/lexer.c"
yy154:
	skip(state);
	goto yy96;
yy155:
	skip(state);
	yych = peek(state);
	if (yych <= 'Z') {
		if (yych <= '/') goto yy156;
		if (yych <= '9') goto yy155;
		if (yych >= 'A') goto yy155;
	} else {
		if (yych <= '_') {
			if (yych >= '_') goto yy155;
		} else {
			if (yych <= '`') goto yy156;
			if (yych <= 'z') goto yy155;
		}
	}
yy156:
#line 131 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tA2IDENT); }
#line 1416 "ext/rbs_extension/lexer.c"
yy157:
	skip(state);
#line 41 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tQIDENT); }
#line 1421 "ext/rbs_extension/lexer.c"
yy158:
	skip(state);
	yych = peek(state);
	if (yych == 'a') goto yy199;
	goto yy52;
yy159:
	skip(state);
	yych = peek(state);
	if (yych == 'r') goto yy200;
	goto yy52;
yy160:
	skip(state);
	yych = peek(state);
	if (yych == 'l') goto yy201;
	goto yy52;
yy161:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy162;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy162;
			if (yych <= 'z') goto yy51;
		}
	}
yy162:
#line 72 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kBOT); }
#line 1459 "ext/rbs_extension/lexer.c"
yy163:
	skip(state);
	yych = peek(state);
	if (yych == 's') goto yy203;
	goto yy52;
yy164:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy165;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy165;
			if (yych <= 'z') goto yy51;
		}
	}
yy165:
#line 74 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kDEF); }
#line 1487 "ext/rbs_extension/lexer.c"
yy166:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy167;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy167;
			if (yych <= 'z') goto yy51;
		}
	}
yy167:
#line 75 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kEND); }
#line 1510 "ext/rbs_extension/lexer.c"
yy168:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy204;
	goto yy52;
yy169:
	skip(state);
	yych = peek(state);
	if (yych == 's') goto yy205;
	goto yy52;
yy170:
	skip(state);
	yych = peek(state);
	if (yych == 'l') goto yy206;
	goto yy52;
yy171:
	skip(state);
	yych = peek(state);
	if (yych == 't') goto yy207;
	goto yy52;
yy172:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy208;
	goto yy52;
yy173:
	skip(state);
	yych = peek(state);
	if (yych == 'u') goto yy209;
	goto yy52;
yy174:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy175;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy175;
			if (yych <= 'z') goto yy51;
		}
	}
yy175:
#line 83 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kNIL); }
#line 1563 "ext/rbs_extension/lexer.c"
yy176:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy177;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy177;
			if (yych <= 'z') goto yy51;
		}
	}
yy177:
#line 84 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kOUT); }
#line 1586 "ext/rbs_extension/lexer.c"
yy178:
	skip(state);
	yych = peek(state);
	if (yych == 'p') goto yy210;
	goto yy52;
yy179:
	skip(state);
	yych = peek(state);
	if (yych == 'v') goto yy211;
	goto yy52;
yy180:
	skip(state);
	yych = peek(state);
	if (yych == 'l') goto yy212;
	goto yy52;
yy181:
	skip(state);
	yych = peek(state);
	if (yych == 'f') goto yy213;
	goto yy52;
yy182:
	skip(state);
	yych = peek(state);
	if (yych == 'g') goto yy215;
	goto yy52;
yy183:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy184;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy184;
			if (yych <= 'z') goto yy51;
		}
	}
yy184:
#line 90 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kTOP); }
#line 1634 "ext/rbs_extension/lexer.c"
yy185:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy216;
	goto yy52;
yy186:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy218;
	goto yy52;
yy187:
	skip(state);
	yych = peek(state);
	if (yych == 'h') goto yy220;
	goto yy52;
yy188:
	skip(state);
	yych = peek(state);
	if (yych == 'y') goto yy221;
	goto yy52;
yy189:
	skip(state);
	yych = peek(state);
	if (yych == 'd') goto yy222;
	goto yy52;
yy190:
	skip(state);
#line 55 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tANNOTATION); }
#line 1664 "ext/rbs_extension/lexer.c"
yy191:
	skip(state);
#line 58 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tANNOTATION); }
#line 1669 "ext/rbs_extension/lexer.c"
yy192:
	skip(state);
#line 56 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tANNOTATION); }
#line 1674 "ext/rbs_extension/lexer.c"
yy193:
	skip(state);
#line 54 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tANNOTATION); }
#line 1679 "ext/rbs_extension/lexer.c"
yy194:
	skip(state);
#line 57 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tANNOTATION); }
#line 1684 "ext/rbs_extension/lexer.c"
yy195:
	yyaccept = 6;
	skip(state);
	backup = *state;
	yych = peek(state);
	if (yych <= '\'') {
		if (yych <= 0x00000000) goto yy147;
		if (yych <= '&') goto yy87;
		goto yy146;
	} else {
		if (yych == '\\') goto yy148;
		goto yy87;
	}
yy196:
	skip(state);
	yych = peek(state);
	if (yych <= '>') {
		if (yych <= '/') {
			if (yych == '!') goto yy224;
		} else {
			if (yych <= '9') goto yy196;
			if (yych == '=') goto yy224;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '?') goto yy224;
			if (yych <= '@') goto yy197;
			if (yych <= 'Z') goto yy196;
		} else {
			if (yych == '`') goto yy197;
			if (yych <= 'z') goto yy196;
		}
	}
yy197:
#line 118 "ext/rbs_extension/lexer.re"
	{ return next_token(state, tSYMBOL); }
#line 1721 "ext/rbs_extension/lexer.c"
yy198:
	skip(state);
	goto yy153;
yy199:
	skip(state);
	yych = peek(state);
	if (yych == 's') goto yy225;
	goto yy52;
yy200:
	skip(state);
	yych = peek(state);
	if (yych == '_') goto yy227;
	goto yy52;
yy201:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy202;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy202;
			if (yych <= 'z') goto yy51;
		}
	}
yy202:
#line 71 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kBOOL); }
#line 1757 "ext/rbs_extension/lexer.c"
yy203:
	skip(state);
	yych = peek(state);
	if (yych == 's') goto yy228;
	goto yy52;
yy204:
	skip(state);
	yych = peek(state);
	if (yych == 'n') goto yy230;
	goto yy52;
yy205:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy231;
	goto yy52;
yy206:
	skip(state);
	yych = peek(state);
	if (yych == 'u') goto yy233;
	goto yy52;
yy207:
	skip(state);
	yych = peek(state);
	if (yych == 'a') goto yy234;
	goto yy52;
yy208:
	skip(state);
	yych = peek(state);
	if (yych == 'r') goto yy235;
	goto yy52;
yy209:
	skip(state);
	yych = peek(state);
	if (yych == 'l') goto yy236;
	goto yy52;
yy210:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy237;
	goto yy52;
yy211:
	skip(state);
	yych = peek(state);
	if (yych == 'a') goto yy238;
	goto yy52;
yy212:
	skip(state);
	yych = peek(state);
	if (yych == 'i') goto yy239;
	goto yy52;
yy213:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy214;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy214;
			if (yych <= 'z') goto yy51;
		}
	}
yy214:
#line 88 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kSELF); }
#line 1830 "ext/rbs_extension/lexer.c"
yy215:
	skip(state);
	yych = peek(state);
	if (yych == 'l') goto yy240;
	goto yy52;
yy216:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy217;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy217;
			if (yych <= 'z') goto yy51;
		}
	}
yy217:
#line 91 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kTRUE); }
#line 1858 "ext/rbs_extension/lexer.c"
yy218:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy219;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy219;
			if (yych <= 'z') goto yy51;
		}
	}
yy219:
#line 92 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kTYPE); }
#line 1881 "ext/rbs_extension/lexer.c"
yy220:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy241;
	goto yy52;
yy221:
	skip(state);
	yych = peek(state);
	if (yych == 'p') goto yy242;
	goto yy52;
yy222:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy223;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy223;
			if (yych <= 'z') goto yy51;
		}
	}
yy223:
#line 95 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kVOID); }
#line 1914 "ext/rbs_extension/lexer.c"
yy224:
	skip(state);
	goto yy197;
yy225:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy226;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy226;
			if (yych <= 'z') goto yy51;
		}
	}
yy226:
#line 67 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kALIAS); }
#line 1940 "ext/rbs_extension/lexer.c"
yy227:
	skip(state);
	yych = peek(state);
	if (yych <= 'q') {
		if (yych == 'a') goto yy243;
		goto yy52;
	} else {
		if (yych <= 'r') goto yy244;
		if (yych == 'w') goto yy245;
		goto yy52;
	}
yy228:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy229;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy229;
			if (yych <= 'z') goto yy51;
		}
	}
yy229:
#line 73 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kCLASS); }
#line 1974 "ext/rbs_extension/lexer.c"
yy230:
	skip(state);
	yych = peek(state);
	if (yych == 'd') goto yy246;
	goto yy52;
yy231:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy232;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy232;
			if (yych <= 'z') goto yy51;
		}
	}
yy232:
#line 77 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kFALSE); }
#line 2002 "ext/rbs_extension/lexer.c"
yy233:
	skip(state);
	yych = peek(state);
	if (yych == 'd') goto yy248;
	goto yy52;
yy234:
	skip(state);
	yych = peek(state);
	if (yych == 'n') goto yy249;
	goto yy52;
yy235:
	skip(state);
	yych = peek(state);
	if (yych == 'f') goto yy250;
	goto yy52;
yy236:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy251;
	goto yy52;
yy237:
	skip(state);
	yych = peek(state);
	if (yych == 'n') goto yy253;
	goto yy52;
yy238:
	skip(state);
	yych = peek(state);
	if (yych == 't') goto yy254;
	goto yy52;
yy239:
	skip(state);
	yych = peek(state);
	if (yych == 'c') goto yy255;
	goto yy52;
yy240:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy257;
	goto yy52;
yy241:
	skip(state);
	yych = peek(state);
	if (yych == 'c') goto yy258;
	goto yy52;
yy242:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy259;
	goto yy52;
yy243:
	skip(state);
	yych = peek(state);
	if (yych == 'c') goto yy260;
	goto yy52;
yy244:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy261;
	goto yy52;
yy245:
	skip(state);
	yych = peek(state);
	if (yych == 'r') goto yy262;
	goto yy52;
yy246:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy247;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy247;
			if (yych <= 'z') goto yy51;
		}
	}
yy247:
#line 76 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kEXTEND); }
#line 2090 "ext/rbs_extension/lexer.c"
yy248:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy263;
	goto yy52;
yy249:
	skip(state);
	yych = peek(state);
	if (yych == 'c') goto yy265;
	goto yy52;
yy250:
	skip(state);
	yych = peek(state);
	if (yych == 'a') goto yy266;
	goto yy52;
yy251:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy252;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy252;
			if (yych <= 'z') goto yy51;
		}
	}
yy252:
#line 82 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kMODULE); }
#line 2128 "ext/rbs_extension/lexer.c"
yy253:
	skip(state);
	yych = peek(state);
	if (yych == 'd') goto yy267;
	goto yy52;
yy254:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy269;
	goto yy52;
yy255:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy256;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy256;
			if (yych <= 'z') goto yy51;
		}
	}
yy256:
#line 87 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kPUBLIC); }
#line 2161 "ext/rbs_extension/lexer.c"
yy257:
	skip(state);
	yych = peek(state);
	if (yych == 't') goto yy271;
	goto yy52;
yy258:
	skip(state);
	yych = peek(state);
	if (yych == 'k') goto yy272;
	goto yy52;
yy259:
	skip(state);
	yych = peek(state);
	if (yych == 'd') goto yy273;
	goto yy52;
yy260:
	skip(state);
	yych = peek(state);
	if (yych == 'c') goto yy275;
	goto yy52;
yy261:
	skip(state);
	yych = peek(state);
	if (yych == 'a') goto yy276;
	goto yy52;
yy262:
	skip(state);
	yych = peek(state);
	if (yych == 'i') goto yy277;
	goto yy52;
yy263:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy264;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy264;
			if (yych <= 'z') goto yy51;
		}
	}
yy264:
#line 79 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kINCLUDE); }
#line 2214 "ext/rbs_extension/lexer.c"
yy265:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy278;
	goto yy52;
yy266:
	skip(state);
	yych = peek(state);
	if (yych == 'c') goto yy280;
	goto yy52;
yy267:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy268;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy268;
			if (yych <= 'z') goto yy51;
		}
	}
yy268:
#line 85 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kPREPEND); }
#line 2247 "ext/rbs_extension/lexer.c"
yy269:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy270;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy270;
			if (yych <= 'z') goto yy51;
		}
	}
yy270:
#line 86 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kPRIVATE); }
#line 2270 "ext/rbs_extension/lexer.c"
yy271:
	skip(state);
	yych = peek(state);
	if (yych == 'o') goto yy281;
	goto yy52;
yy272:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy282;
	goto yy52;
yy273:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy274;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy274;
			if (yych <= 'z') goto yy51;
		}
	}
yy274:
#line 94 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kUNTYPED); }
#line 2303 "ext/rbs_extension/lexer.c"
yy275:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy283;
	goto yy52;
yy276:
	skip(state);
	yych = peek(state);
	if (yych == 'd') goto yy284;
	goto yy52;
yy277:
	skip(state);
	yych = peek(state);
	if (yych == 't') goto yy285;
	goto yy52;
yy278:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy279;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy279;
			if (yych <= 'z') goto yy51;
		}
	}
yy279:
#line 80 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kINSTANCE); }
#line 2341 "ext/rbs_extension/lexer.c"
yy280:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy286;
	goto yy52;
yy281:
	skip(state);
	yych = peek(state);
	if (yych == 'n') goto yy288;
	goto yy52;
yy282:
	skip(state);
	yych = peek(state);
	if (yych == 'd') goto yy290;
	goto yy52;
yy283:
	skip(state);
	yych = peek(state);
	if (yych == 's') goto yy292;
	goto yy52;
yy284:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy293;
	goto yy52;
yy285:
	skip(state);
	yych = peek(state);
	if (yych == 'e') goto yy294;
	goto yy52;
yy286:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy287;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy287;
			if (yych <= 'z') goto yy51;
		}
	}
yy287:
#line 81 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kINTERFACE); }
#line 2394 "ext/rbs_extension/lexer.c"
yy288:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy289;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy289;
			if (yych <= 'z') goto yy51;
		}
	}
yy289:
#line 89 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kSINGLETON); }
#line 2417 "ext/rbs_extension/lexer.c"
yy290:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy291;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy291;
			if (yych <= 'z') goto yy51;
		}
	}
yy291:
#line 93 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kUNCHECKED); }
#line 2440 "ext/rbs_extension/lexer.c"
yy292:
	skip(state);
	yych = peek(state);
	if (yych == 's') goto yy295;
	goto yy52;
yy293:
	skip(state);
	yych = peek(state);
	if (yych == 'r') goto yy296;
	goto yy52;
yy294:
	skip(state);
	yych = peek(state);
	if (yych == 'r') goto yy298;
	goto yy52;
yy295:
	skip(state);
	yych = peek(state);
	if (yych == 'o') goto yy300;
	goto yy52;
yy296:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy297;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy297;
			if (yych <= 'z') goto yy51;
		}
	}
yy297:
#line 69 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kATTRREADER); }
#line 2483 "ext/rbs_extension/lexer.c"
yy298:
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy299;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy299;
			if (yych <= 'z') goto yy51;
		}
	}
yy299:
#line 70 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kATTRWRITER); }
#line 2506 "ext/rbs_extension/lexer.c"
yy300:
	skip(state);
	yych = peek(state);
	if (yych != 'r') goto yy52;
	skip(state);
	yych = peek(state);
	if (yych <= '=') {
		if (yych <= '/') {
			if (yych == '!') goto yy104;
		} else {
			if (yych <= '9') goto yy51;
			if (yych >= '=') goto yy105;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy301;
			if (yych <= 'Z') goto yy51;
		} else {
			if (yych == '`') goto yy301;
			if (yych <= 'z') goto yy51;
		}
	}
yy301:
#line 68 "ext/rbs_extension/lexer.re"
	{ return next_token(state, kATTRACCESSOR); }
#line 2532 "ext/rbs_extension/lexer.c"
}
#line 140 "ext/rbs_extension/lexer.re"

}
